@@ -4,8 +4,9 @@ use Mojo::Base 'Mojo::EventEmitter';
 use Mojo::IOLoop;
 use Mojo::IOLoop::Delay;
 use Mojo::SMTP::Client::Exception;
+use Carp;
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 use constant {
 	CMD_OK       => 2,
@@ -18,7 +19,8 @@ use constant {
 	CMD_TO       => 5,
 	CMD_DATA     => 6,
 	CMD_DATA_END => 7,
-	CMD_QUIT     => 8,
+	CMD_RESET    => 8,
+	CMD_QUIT     => 9,
 	
 	CRLF         => "\x0d\x0a",
 };
@@ -31,6 +33,7 @@ our %CMD = (
 	&CMD_TO       => 'CMD_TO',
 	&CMD_DATA     => 'CMD_DATA',
 	&CMD_DATA_END => 'CMD_DATA_END',
+	&CMD_RESET    => 'CMD_RESET',
 	&CMD_QUIT     => 'CMD_QUIT',
 );
 
@@ -42,10 +45,18 @@ has inactivity_timeout => sub { $ENV{MOJO_INACTIVITY_TIMEOUT} // 20 };
 has ioloop             => sub { Mojo::IOLoop->new };
 has autodie            => 0;
 
+my %cmd = (
+	from  => 1,
+	to    => 1,
+	data  => 1,
+	reset => 1,
+	quit  => 1,
+);
+
 sub send {
 	my $self = shift;
 	my $cb = @_ % 2 == 0 ? undef : pop;
-	my %cmd = @_;
+	my @cmd = @_;
 	
 	my @steps;
 	my $expected_code;
@@ -115,114 +126,118 @@ sub send {
 		$self->{stream}->start;
 	}
 	
-	if (exists $cmd{from}) {
-		# FROM
-		my $from = delete $cmd{from};
-		push @steps, sub {
-			my $delay = shift;
-			$self->_cmd('MAIL FROM:<'.$from.'>', CMD_FROM);
-			$self->_read_response($delay->begin);
-			$expected_code = CMD_OK;
-		},
-		$resp_checker
-	}
-	if (exists $cmd{to}) {
-		# TO
-		for my $to (ref $cmd{to} ? @{$cmd{to}} : $cmd{to}) {
+	for (my $i=0; $i<@cmd; $i+=2) {
+		my $mi = $i+1;
+		
+		if ($cmd[$i] eq 'from') { # FROM
 			push @steps, sub {
 				my $delay = shift;
-				$self->_cmd('RCPT TO:<'.$to.'>', CMD_TO);
+				$self->_cmd('MAIL FROM:<'.$cmd[$mi].'>', CMD_FROM);
 				$self->_read_response($delay->begin);
 				$expected_code = CMD_OK;
 			},
 			$resp_checker
 		}
-		delete $cmd{to};
-	}
-	if (exists $cmd{data}) {
-		# DATA
-		push @steps, sub {
-			my $delay = shift;
-			$self->_cmd('DATA', CMD_DATA);
-			$self->_read_response($delay->begin);
-			$expected_code = CMD_MORE;
-		},
-		$resp_checker;
-		
-		if (ref $cmd{data} eq 'CODE') {
-			my ($data_writer, $data_writer_cb);
-			my $data_generator = delete $cmd{data};
-			my $was_nl;
-			
-			$data_writer = sub {
-				my $delay = shift;
-				unless ($data_writer_cb) {
-					$data_writer_cb = $delay->begin;
-					$self->_set_errors_handler(sub {
-						$data_writer_cb->($delay, @_);
-						undef $data_writer;
-					});
-				}
-				
-				my $data = $data_generator->();
-				
-				unless (length(ref $data ? $$data : $data) > 0) {
-					$self->_cmd(($was_nl ? '' : CRLF).'.', CMD_DATA_END);
-					$self->_read_response($data_writer_cb);
-					$self->_set_errors_handler(undef);
+		elsif ($cmd[$i] eq 'to') { # TO
+			for my $to (ref $cmd[$mi] ? @{$cmd[$mi]} : $cmd[$mi]) {
+				push @steps, sub {
+					my $delay = shift;
+					$self->_cmd('RCPT TO:<'.$to.'>', CMD_TO);
+					$self->_read_response($delay->begin);
 					$expected_code = CMD_OK;
-					return undef $data_writer;
-				}
-				
-				$was_nl = _has_nl($data);
-				$self->{stream}->write(ref $data ? $$data : $data, $data_writer);
-			};
+				},
+				$resp_checker
+			}
+		}
+		elsif ($cmd[$i] eq 'data') { # DATA
+			# DATA
+			push @steps, sub {
+				my $delay = shift;
+				$self->_cmd('DATA', CMD_DATA);
+				$self->_read_response($delay->begin);
+				$expected_code = CMD_MORE;
+			},
+			$resp_checker;
 			
-			push @steps, $data_writer, $resp_checker;
+			if (ref $cmd[$mi] eq 'CODE') {
+				my ($data_writer, $data_writer_cb);
+				my $was_nl;
+				
+				$data_writer = sub {
+					my $delay = shift;
+					unless ($data_writer_cb) {
+						$data_writer_cb = $delay->begin;
+						$self->_set_errors_handler(sub {
+							$data_writer_cb->($delay, @_);
+							undef $data_writer;
+						});
+					}
+					
+					my $data = $cmd[$mi]->();
+					
+					unless (length(ref $data ? $$data : $data) > 0) {
+						$self->_cmd(($was_nl ? '' : CRLF).'.', CMD_DATA_END);
+						$self->_read_response($data_writer_cb);
+						$self->_set_errors_handler(undef);
+						$expected_code = CMD_OK;
+						return undef $data_writer;
+					}
+					
+					$was_nl = _has_nl($data);
+					$self->{stream}->write(ref $data ? $$data : $data, $data_writer);
+				};
+				
+				push @steps, $data_writer, $resp_checker;
+			}
+			else {
+				push @steps, sub {
+					my $delay = shift;
+					my $data_writer_cb = $delay->begin;
+					$self->{stream}->write(ref $cmd[$mi] ? ${$cmd[$mi]} : $cmd[$mi], $data_writer_cb);
+					$self->_set_errors_handler(sub {
+						$data_writer_cb->(@_);
+					});
+				},
+				sub {
+					my ($delay, $resp) = @_;
+					if ($resp && $resp->{error}) {
+						die $resp->{error};
+					}
+					
+					$self->_set_errors_handler(undef);
+					$self->_cmd((_has_nl($cmd[$mi]) ? '' : CRLF).'.', CMD_DATA_END);
+					$self->_read_response($delay->begin);
+					$expected_code = CMD_OK;
+				},
+				$resp_checker
+			}
+		}
+		elsif ($cmd[$i] eq 'reset') { # RESET
+			push @steps, sub {
+				my $delay = shift;
+				$self->_cmd('RSET', CMD_RESET);
+				$self->_read_response($delay->begin);
+				$expected_code = CMD_OK;
+			},
+			$resp_checker
+		}
+		elsif ($cmd[$i] eq 'quit') { # QUIT
+			push @steps, sub {
+				my $delay = shift;
+				$self->_cmd('QUIT', CMD_QUIT);
+				$self->_read_response($delay->begin);
+				$expected_code = CMD_OK;
+			},
+			$resp_checker, sub {
+				my $delay = shift;
+				delete($self->{stream})->close;
+				$delay->pass(@_);
+			}
 		}
 		else {
-			my $data = delete $cmd{data};
-			push @steps, sub {
-				my $delay = shift;
-				my $data_writer_cb = $delay->begin;
-				$self->{stream}->write(ref $data ? $$data : $data, $data_writer_cb);
-				$self->_set_errors_handler(sub {
-					$data_writer_cb->(@_);
-				});
-			},
-			sub {
-				my ($delay, $resp) = @_;
-				if ($resp && $resp->{error}) {
-					die $resp->{error};
-				}
-				
-				$self->_set_errors_handler(undef);
-				$self->_cmd((_has_nl($data) ? '' : CRLF).'.', CMD_DATA_END);
-				$self->_read_response($delay->begin);
-				$expected_code = CMD_OK;
-			},
-			$resp_checker
+			croak 'unrecognized command: ', $cmd[$i];
 		}
 	}
-	if (exists $cmd{quit}) {
-		# QUIT
-		delete $cmd{quit};
-		push @steps, sub {
-			my $delay = shift;
-			$self->_cmd('QUIT', CMD_QUIT);
-			$self->_read_response($delay->begin);
-			$expected_code = CMD_OK;
-		},
-		$resp_checker, sub {
-			my $delay = shift;
-			delete($self->{stream})->close;
-			$delay->pass(@_);
-		};
-	}
-	
-	if (%cmd) {
-		die "unrecognized commands specified: ", join(", ", keys %cmd);
-	}
 	
 	# non-blocking
 	my $delay = Mojo::IOLoop::Delay->new(ioloop => $self->_ioloop($nb))->steps(@steps)->catch(sub {
@@ -331,6 +346,8 @@ __END__
 
 =pod
 
+=encoding utf8
+
 =head1 NAME
 
 Mojo::SMTP::Client - non-blocking SMTP client based on Mojo::IOLoop
@@ -486,6 +503,13 @@ or reference to array with email strings (for more than one recipient)
 	$smtp->send(to => 'oleg@cpan.org');
 	$smtp->send(to => ['oleg@cpan.org', 'do_not_reply@cpantesters.org']);
 
+=item reset
+
+After this command server should forget about any started mail transaction and reset it status as it was after response to C<EHLO>/C<HELO>.
+Note: transaction considered started after C<MAIL FROM> (C<from>) command.
+
+	$smtp->send(reset => 1);
+
 =item data
 
 Email body to be sent. Value for this command should be a string (or reference to a string) with email body or reference to subroutine
@@ -515,6 +539,21 @@ For blocking usage C<$resp> will be returned as result of C<$smtp-E<gt>send> cal
 non-blocking result. If L</autodie> attribute has true value C<send> will throw an exception on any error.
 Which will be one of C<Mojo::SMTP::Client::Exception::*>.
 
+B<Note>. For SMTP protocol it is important to send commands in certain order. Also C<send> will send all commands in order you are
+specified. So, it is important to pass arguments to C<send> in right order. For basic usage this will always be:
+C<from -E<gt> to -E<gt> data -E<gt> quit>. You should also know that it is absolutely correct to specify several non-unique commands.
+For example you can send several emails with one C<send> call:
+
+	$smtp->send(
+		from => 'someone@somewhere.com',
+		to   => 'somebody@somewhere.net',
+		data => $mail_1,
+		from => 'frodo@somewhere.com',
+		to   => 'garry@somewhere.net',
+		data => $mail_2,
+		quit => 1
+	);
+
 B<Note>. Connection to SMTP server will be made on first C<send> or for each C<send> when socket connection not already estabilished
 (was closed by C<QUIT> command or errors in the stream). It is error to make several simultaneous non-blocking C<send> calls on the
 same C<Mojo::SMTP::Client>, because each client has one global stream per client. So, you need to create several
@@ -531,6 +570,7 @@ C<Mojo::SMTP::Client> has this non-importable constants
 	CMD_TO       # client sent RCPT TO command
 	CMD_DATA     # client sent DATA command
 	CMD_DATA_END # client sent . command
+	CMD_RESET    # client sent RSET command
 	CMD_QUIT     # client sent QUIT command
 
 =head1 VARIABLES
@@ -551,7 +591,7 @@ Get human readable command by it constant
 
 =head2 How to send simple ASCII message
 
-ASCII message is simple enough, so you can generate it by hands
+ASCII message is simple enough, so you can generate it by hand
 
 	$smtp->send(
 		from => 'me@home.org',
@@ -574,7 +614,7 @@ familar with MIME standard. For more convenient approaches see below.
 =head2 How to send text message with possible non-ASCII characters
 
 For more convinient way to generate emails we can use some email generators
-available on CPAN. C<MIME::Lite> for example. With such modules we can get
+available on CPAN. L<MIME::Lite> for example. With such modules we can get
 email as a string and send it with C<Mojo::SMTP::Client>
 
 	use MIME::Lite;
@@ -598,7 +638,7 @@ email as a string and send it with C<Mojo::SMTP::Client>
 
 =head2 How to send message with attachment
 
-This is also simple with help of C<MIME::Lite>
+This is also simple with help of L<MIME::Lite>
 
 	use MIME::Lite;
 	
@@ -619,7 +659,7 @@ This is also simple with help of C<MIME::Lite>
 =head2 How to send message with BIG attachment
 
 It will be not cool to get message with 50 mb attachment into memory before sending.
-Fortunetly with help of C<MIME::Lite> and C<MIME::Lite::Generator> we can generate
+Fortunately with help of L<MIME::Lite> and L<MIME::Lite::Generator> we can generate
 our email by small portions. As you remember C<data> command accepts subroutine reference
 as argument, so it will be super easy to send our big email in memory-efficient way
 
@@ -642,12 +682,12 @@ as argument, so it will be super easy to send our big email in memory-efficient
 		data => sub { $generator->get() }
 	);
 
-=head2 How to send message directly without using of MTA such as sendmail, postfix, exim, ...
+=head2 How to send message directly, without using of MTAs such as sendmail, postfix, exim, ...
 
 Sometimes it is more suitable to send message directly to SMTP server of recipient. For example
 if you haven't any MTA available or want to check recipient's server responses (e.g. to know is
 such user exists on this server). First you need to know address of necessary SMTP server. We'll
-get it with help of C<Net::DNS>. Then we'll send it as usual
+get it with help of L<Net::DNS>. Then we'll send it as usual
 
 	# will use non-blocking approach in this example
 	use strict;
@@ -680,7 +720,7 @@ get it with help of C<Net::DNS>. Then we'll send it as usual
 		
 		my $smtp = Mojo::SMTP::Client->new(
 			address => $address,
-			# it is important properly identify yourself
+			# it is important to properly identify yourself
 			hello   => 'home.org'
 		);
 		
